#include "Feld.h"
#include "Ressourcen.h"
#include <istream>


void Feld::draw(sf::RenderTarget& Target, sf::RenderStates states) const {
	Target.draw(sp, states);
}

void Feld::redraw() {
	texture.clear(sf::Color::Transparent);
	for(uint8_t i = 0; i < 15; i++)
		for(uint8_t j = 0; j < 15; j++)
			if(Steine[i][j] != nullptr)
				texture.draw(*Steine[i][j]);
	texture.display();
}

Feld::Feld()
	: SteinVerschwindet(RM().get<sf::SoundBuffer>("Sound/Verschwindet.wav"))
{
	memset(Steine, 0, 15*15*sizeof(Stein*));
	texture.create(1024, 768);
	texture.clear(sf::Color::Transparent);
	sp.setTexture(texture.getTexture(), true);
}

void Feld::Load(std::istream& ifs) {
	Unload();
	for(uint8_t j = 0; j < 15; j++) {
		for(uint8_t i = 0; i < 15; i++) {
			char typ;
			ifs >> typ;
			switch(typ) {
				case 'F':
					Steine[i][j] = new Stein(Stein::FEST);
					break;
				case 'P':
					Steine[i][j] = new Stein(Stein::POSITIV);
					break;
				case 'N':
					Steine[i][j] = new Stein(Stein::NEGATIV);
					break;
				default:
					break;
			}
			if(Steine[i][j] != 0)
				Steine[i][j]->setPosition(167.f + i*46.f, 129.f + j*34.f);
		}
	}
	redraw();
}

void Feld::Unload() {
	for(uint8_t i = 0; i < 15; i++) {
		for(uint8_t j = 0; j < 15; j++) {
			delete Steine[i][j];
			Steine[i][j] = 0;
		}
	}
}

int16_t Feld::Destroy(uint8_t i, uint8_t j) {
	switch(Steine[i][j]->GetTyp()) {
		case Stein::POSITIV:
			delete Steine[i][j];
			Steine[i][j] = 0;
			SteinVerschwindet.play();
			redraw();
			return(1);
		case Stein::NEGATIV:
			delete Steine[i][j];
			Steine[i][j] = 0;
			SteinVerschwindet.play();
			redraw();
			return(-1);
		default:
			return(0);
	}
}

bool Feld::Abgerumt() const {
	for(uint8_t i = 0; i < 15; i++) {
		for(uint8_t j = 0; j < 15; j++) {
			if(Steine[i][j] && Steine[i][j]->GetTyp() == Stein::POSITIV)
				return(false);
		}
	}
	return(true);
}

Feld::~Feld() {
	Unload();
}
