#include "Spiel.h"
#include "Ressourcen.h"
#include <fstream>


bool operator>(const sf::FloatRect& A, const sf::FloatRect& B) {
	return(A.height+A.width > B.height+B.width); // Umfang besser als Flcheninhalt
}

void Spiel::draw(sf::RenderTarget& Target, sf::RenderStates states) const {
	Target.draw(F);
	Target.draw(B);
	for(uint8_t i = 0; i < 4; i++) {
		Target.draw(S[i], states);
	}
}

Spiel::Spiel() : Level(0), Punkte(0) {
	S[0].setRotation(90);
	S[1].setRotation(180);
	S[2].setRotation(270);

	std::ifstream ifs("../../Daten/Breakout/Level/Status");
	if(ifs.good()) {
		ifs >> Level;
		ifs.close();
	}

	NextLevel();
}

void Spiel::NextLevel() {
	std::ifstream ifs("../../Daten/Breakout/Level/" + std::to_string(Level) + ".bs");
	
	if(ifs.good()) {
		SetStatus(BEGINN);
		B.setPosition(75, 384);
		F.Load(ifs);
		ifs.close();
	}
	else { // Spiel durchgespielt.
		SetStatus(ENDE);
	}
}

void Spiel::Run(float Time, const sf::Vector2i& MousePos) {
	static const int x = RM().get<sf::Texture>("Grafik/Schlaeger.png").getSize().x/2;
	static const int y = RM().get<sf::Texture>("Grafik/Schlaeger.png").getSize().y/2;
	S[0].setPosition((float)clamp(y+x*2, MousePos.x, 1024-y-x*2), (float)x, Time);
	S[2].setPosition((float)clamp(y+x*2, MousePos.x, 1024-y-x*2), (float)(768 - x), Time);
	S[1].setPosition((float)x, (float)clamp(y, MousePos.y, 768-y), Time);
	S[3].setPosition((float)(1024 - x), (float)clamp(y, MousePos.y, 768-y), Time);

	if(GetStatus() == SPIEL) {
		B.move(Time);

		// Wand
		if(B.KollisionW(sf::FloatRect(0.f, 0.f, 1024.f, 768.f))) {
			B.Wand.play();
			Punkte--;
		}
		else {
			// Schlger
			sf::FloatRect Ball = B.getLocalBounds();
			if(Ball.intersects(S[0].getLocalBounds())) {
				B.Bewegung.Alpha = 90.f-1.25f*((Ball.left+Ball.width/2.f) - (S[0].getLocalBounds().left + S[0].getLocalBounds().width/2.f));
				B.Schlger.play();
			}
			else if(Ball.intersects(S[2].getLocalBounds())) {
				B.Bewegung.Alpha = -90.f+1.25f*((Ball.left+Ball.width/2.f) - (S[2].getLocalBounds().left + S[2].getLocalBounds().width/2.f));
				B.Schlger.play();
			}
			if(Ball.intersects(S[1].getLocalBounds())) {
				B.Bewegung.Alpha = 1.25f*((Ball.top+Ball.height/2.f) - (S[1].getLocalBounds().top + S[1].getLocalBounds().height/2.f));
				B.Schlger.play();
			}
			else if(Ball.intersects(S[3].getLocalBounds())) {
				B.Bewegung.Alpha = 180.f-1.25f*((Ball.top+Ball.height/2.f) - (S[3].getLocalBounds().top + S[3].getLocalBounds().height/2.f));
				B.Schlger.play();
			}

			// Steine
			int8_t s_x = -1, s_y = -1;
			sf::FloatRect Grtes(0,0,0,0);
			sf::FloatRect Temp;
			for(uint8_t i = 0; i < 15; i++) { // Strkste berschneidung rausfinden
				for(uint8_t j = 0; j < 15; j++) {
					if(F.Steine[i][j]) {
						Ball.intersects(F.Steine[i][j]->getLocalBounds(), Temp);
						if(Temp>Grtes) {
							Grtes = Temp;
							s_x = i;
							s_y = j;
						}
					}
				}
			}

			if(s_x != -1 && s_y != -1 && B.KollisionI(F.Steine[s_x][s_y]->getLocalBounds())) {
				Punkte += F.Destroy(s_x, s_y);
				B.Stein.play();
			}
		}

		if(F.Abgerumt()) {
			if(Punkte > 0)
				Level++;
			NextLevel();
		}
	}
}

Spiel::~Spiel() {
	std::ofstream ofs("../../Daten/Breakout/Level/Status");
	if(ofs.good()) {
		ofs << Level;
		ofs.close();
	}
}
