#include "Global.h"
#include "Spielfeld.h"
#include "Spieler.h"
#include "Musik.h"
#include "Checkbox.h"
#include "Button.h"
#include <ctime>
#include <SFML/Window.hpp>
#include <SFML/Graphics.hpp>

bool Running = true;
bool Paused = false;
bool Verloren = false;

void NeuesSpielClicked();

class HextrisBattle : public sf::Drawable {
	virtual void draw(sf::RenderTarget& Target, sf::RenderStates) const {
		Target.draw(Feld);
		Target.draw(SpielerL);
		Target.draw(SpielerR);
	}
	void Run() {
		// Thread steuert zeitliche Abfolge und Teile der Spiellogik
		std::srand(~(static_cast<uint32_t>(std::time(nullptr)))); // ~ fr in jedem Thread "garantiert" verschiedene Seeds (sonst 2 mal gleiche Steine)

		sf::Clock Uhr;
		while(Running) {
			if(SpielerR.Verloren || SpielerL.Verloren) {
				for(int i = 0; i < 100 && Running; ++i) {
					sf::sleep(sf::milliseconds(100));
				}
				Verloren = true;
			}
			else {
				sf::sleep(sf::milliseconds(550 - Uhr.getElapsedTime().asMilliseconds())); // Rechenzeit sparen...
				Uhr.restart();
				if(!Paused) {
					Feld.NextStep();
				}
			}
		}
	}
public:
	Spieler SpielerL;
	Spieler SpielerR;
	Spielfeld Feld;
	sf::Thread Executor;
	HextrisBattle() : SpielerL(LINKS), SpielerR(RECHTS), Feld(SpielerR, SpielerL), Executor(std::bind(&HextrisBattle::Run, this)) {
		Executor.launch();
	}
	void Process() {
		if(Verloren) {
			NeuesSpielClicked();
			return;
		}
		Feld.Protector.lock();
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::Left)) {
			SpielerR.Links();
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::Right)) {
			SpielerR.Rechts();
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::A)) {
			SpielerL.Links();
		}
		if(sf::Keyboard::isKeyPressed(sf::Keyboard::S)) {
			SpielerL.Rechts();
		}
		Feld.Protector.unlock();
	}
	~HextrisBattle() {
		Executor.wait();
	}
};

HextrisBattle* Game;

Player MusicPlayer;

void MusicChecked(bool checked) {
	if(checked)
		MusicPlayer.Start();
	else
		MusicPlayer.Stop();
}
void ExitClicked() {
	Running = false;
	delete Game;
	MusicPlayer.Stop();
	exit(0);
}
void NeuesSpielClicked() {
	Running = false;
	delete Game;
	Running = true;
	Paused = false;
	Verloren = false;
	Game = new HextrisBattle();
}
void PauseChecked(bool checked) {
	Paused = checked;
}
void SteinformationenChecked(bool checked) {
	MultiForm = checked;
}

sf::RenderWindow RW;

void VollbildChecked(bool checked) {
	if(checked) {
		RW.create(Auflsung, "Hextris-Battle 1.0 - Weihnachtsmann-Edition", sf::Style::Fullscreen, sf::ContextSettings(0, 0, 4));
	}
	else {
		RW.create(Auflsung, "Hextris-Battle 1.0 - Weihnachtsmann-Edition", sf::Style::Titlebar | sf::Style::Close, sf::ContextSettings(0, 0, 4));
	}
	RW.setFramerateLimit(60);
}

int main() {
	MusicPlayer.Load("Musik/Nikolaus.ogg");
	MusicPlayer.Load("Musik/Tannebaum.ogg");
	MusicPlayer.Load("Musik/Alle Jahre.ogg");

	// Events und Input
	sf::Event EV;

	// Sprites initiieren
	sf::Sprite Hintergrund(RM().get<sf::Texture>("Grafik/Hintergrund.png"));

	// GUI initiieren
	CheckBox Vollbild("Vollbild", sf::Vector2f(7.f, Auflsung.height - 52.f), &VollbildChecked, LINKS, false);
	CheckBox Music("Musik abspielen", sf::Vector2f(7.f, Auflsung.height - 25.f), &MusicChecked, LINKS);
	Button Exit("Beenden", sf::Vector2f(Auflsung.width/2.f - 110, Auflsung.height - 33.f), sf::Vector2f(95, 25), &ExitClicked);
	Button NeuesSpiel("Neues Spiel", sf::Vector2f(Auflsung.width/2.f, Auflsung.height - 33.f), sf::Vector2f(120, 25), &NeuesSpielClicked);
	CheckBox Pause("Pausiert", sf::Vector2f(Auflsung.width - 7.f, Auflsung.height - 52.f), &PauseChecked, RECHTS, false);
	CheckBox Steinformationen("Steinformationen", sf::Vector2f(Auflsung.width - 7.f, Auflsung.height - 25.f), &SteinformationenChecked, RECHTS, false);

	Game = new HextrisBattle();

	while(Running) {
		while(RW.pollEvent(EV)) {
			switch(EV.type) {
				case sf::Event::Closed:
					RW.close();
					Running = false;
					break;
				case sf::Event::KeyPressed:
					switch(EV.key.code) {
						case sf::Keyboard::Escape:
							RW.close();
							Running = false;
							break;
						case sf::Keyboard::Pause:
							Pause.Check(!Pause.IsChecked());
							break;
						default:
							break;
					}
					break;
				case sf::Event::MouseButtonPressed:
					if(EV.mouseButton.button == sf::Mouse::Left) {
						Music.Clicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y));
						Exit.IsClicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y));
						if(NeuesSpiel.IsClicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y))) {
							Pause.Check(false);
						}
						Pause.Clicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y));
						Vollbild.Clicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y));
						Steinformationen.Clicked(sf::Vector2i(EV.mouseButton.x, EV.mouseButton.y));
					}
					break;
				default:
					break;
			}
		}
		Game->Process();
		RW.clear();
		RW.draw(Hintergrund);
		RW.draw(*Game);
		RW.draw(Music);
		RW.draw(Exit);
		RW.draw(NeuesSpiel);
		RW.draw(Pause);
		RW.draw(Vollbild);
		RW.draw(Steinformationen);
		RW.display();
	}
	ExitClicked();
	return(0);
}
