#include "SpaceShooter.h"
#include "Ressourcen.h"


void SpaceShooter::draw(sf::RenderTarget& Target, sf::RenderStates states) const {
	Target.setView(Ich.Kamera);
	Target.draw(Ich, states);
	for(auto i = gegner.cbegin(), j = gegner.cend(); i != j; ++i) {
		Target.draw(*i, states);
	}
	for(auto i = PowerUps.cbegin(), j = PowerUps.cend(); i != j; ++i) {
		Target.draw(*i, states);
	}
	for(auto i = Projektile.cbegin(), j = Projektile.cend(); i != j; ++i) {
		Target.draw(**i, states);
	}
	Target.draw(Grenze, states);
	Target.setView(Target.getDefaultView());
	Target.draw(mGeschwindigkeit, states);
	Target.draw(mGeschwindigkeitsDir, states);
	Target.draw(aLeben, states);
	Target.draw(aMunition, states);
	Target.draw(aTreibstoff, states);
	Target.draw(sLeben, states);
	Target.draw(sMunition, states);
	Target.draw(sTreibstoff, states);
}

void SpaceShooter::Run(bool Umschalttaste, float Time) {
	// Ich
	Ich.Run(Time);

	// Projektile
	for(auto i = Projektile.begin(), j = Projektile.end(); i != j; ++i) {
		if((*i)->Run(Time)) {
			delete *i;
			i = Projektile.erase(i);
		}
		else if((*i)->Sender != &Ich && Collision(Ich, **i)) {
			Ich.Leben -= (*i)->GetSchaden();
			Punkte -= (int32_t)(*i)->GetSchaden();
			delete *i;
			i = Projektile.erase(i);
		}
		else {
			for(auto k = gegner.begin(), l = gegner.end(); k != l; ++k) {
				if((*i)->Sender != &(*k) && Collision(*k, **i)) {
					k->Leben -= (*i)->GetSchaden();
					if((*i)->Sender == &Ich)
						Punkte += (int32_t)(*i)->GetSchaden();
					delete *i;
					i = Projektile.erase(i);
					break;
				}
			}
		}
		if(i == j) break;
	}
	
	// Gegner
	sf::Vector2f Ziel = Ich.getPosition();
	for(auto i = gegner.begin(), j = gegner.end(); i != j; ++i) {
		if(Collision(*i, Ich)) { // Kollision mit Spieler
			int Abzug = (i->Bewegung - Ich.Bewegung).v*(i->Bewegung - Ich.Bewegung).v;
			Ich.Leben -= Abzug;
			Punkte -= Abzug;
			i = gegner.erase(i);
			Ich.Crash.play();
			if(i == j) break;
		}
		else {
			std::pair<bool, Projektil*> p = i->Run(Time, Ziel);
			if(p.first) {
				i = gegner.erase(i);
				if(i == j) break;
			}
			else if(p.second != nullptr) {
				Projektile.push_back(p.second);
				p.second->Sender = &(*i);
			}
		}
	}
	if((std::rand() % std::max<int>(1, (int)(0.5f+((float)gegner.size())/15.f)/Time)) == 0)
	{
		gegner.emplace_back(Ich.getPosition());
	}

	// PowerUps
	for(auto i = PowerUps.begin(), j = PowerUps.end(); i != j; ++i) {
		if(Collision(*i, Ich)) {
			switch(i->Typ) {
				case PowerUp::LEBEN:
					Ich.Leben = std::min(200.f, Ich.Leben+100.f);
					break;
				case PowerUp::TREIBSTOFF:
					Ich.Treibstoff = std::min(100.f, Ich.Treibstoff+50.f);
					break;
				case PowerUp::DROHNE:
					Ich.Munition.Add(&Projektil::Typ::Drohne, 1);
					break;
				case PowerUp::LASER:
					Ich.Munition.Add(&Projektil::Typ::Laser, 15);
					break;
				case PowerUp::KUGEL:
					Ich.Munition.Add(&Projektil::Typ::Kugel, 50);
					break;
			}
			PowerUps.erase(i);
			PowerUps.emplace_back();
			Ich.PowerUpSound.play();
			break;
		}
	}

	// Men/Steuerung
	if(Umschalttaste) {
		++Ich.Munition;
		sMunition.setTexture(Ich.Munition.GetLogo());
		aMunition.Max = Ich.Munition.GetMax();
		aMunition.ZuBeobachten = Ich.Munition.GetVorat();
	}
	if(sf::Keyboard::isKeyPressed(sf::Keyboard::Space)) {
		Projektil* p = Ich.Munition(Ich.getPosition(), Ich.Bewegung, Ich.getRotation());
		if(p != nullptr) {
			Projektile.push_back(p);
			p->Sender = &Ich;
		}
	}
	mGeschwindigkeit.setString(("v: " + std::to_string((long double)Ich.Bewegung.v).substr(0, 8)));
	mGeschwindigkeitsDir.setRotation(-Ich.Bewegung.Alpha);
}

SpaceShooter::SpaceShooter()
	: Ich(), aLeben(&Ich.Leben, 200.f, sf::FloatRect(40,738,150,15)), aMunition(Ich.Munition.GetVorat(), Ich.Munition.GetMax(), sf::FloatRect(40,711,150,15)), aTreibstoff(&Ich.Treibstoff, 100.f, sf::FloatRect(40,684,150,15)),
	  sLeben(RM().get<sf::Texture>("Grafik/Leben.png")), sMunition(Ich.Munition.GetLogo()), sTreibstoff(RM().get<sf::Texture>("Grafik/Treibstoff.png")),
	  Feldgrenze(-5000,-5000,10000,10000), Grenze(sf::RectangleShape(sf::Vector2f(Feldgrenze.width, Feldgrenze.height)))
	  , mGeschwindigkeit("v: 0", RM().get<sf::Font>("../Fonts/sansation.ttf")), mGeschwindigkeitsDir(sf::FloatRect(920,633,75,75))
{
	Grenze.setPosition(-5000, -5000);
	Grenze.setFillColor(sf::Color::Transparent);
	Grenze.setOutlineColor(sf::Color(225,25,25,200));
	Grenze.setOutlineThickness(3);
	sLeben.setPosition(5, 735);
	sMunition.setPosition(5, 704);
	sTreibstoff.setPosition(8, 678);
	for(uint16_t i = 0; i < 200; i++) {
		PowerUps.emplace_back();
	}
	mGeschwindigkeit.setPosition(900, 723);
	mGeschwindigkeit.setColor(sf::Color(50,225,50,200));
}

SpaceShooter::~SpaceShooter() {
	for(auto i = Projektile.cbegin(), j = Projektile.cend(); i != j; ++i) {
		delete *i;
	}
}
