#include "Shell.h"
#include "Status.h"
#include <iostream>
#include <fstream>



Shell::Shell(const char* folder, const char* Welcome, const char* prompt)
	: Prompt(prompt), Folder(folder), Thr(std::bind(&Shell::Run, this))
{
	std::cout << Welcome << std::endl << std::endl;
	Thr.launch();
}

Shell::~Shell() {
	Thr.terminate();
}

void Shell::Run() const {
	std::string command;
	for(;;) {
		std::cout << std::endl << std::endl << Prompt;

		std::cin >> command;

		(*this)(command);
	}
}

void Shell::displayText(const std::string& name) const {
	std::ifstream ifs(Folder + "/" + name + ".txt");
	if(!ifs.good()) throw 0; // File not found -> unknown text

	std::string Text = std::string(std::istreambuf_iterator<char>(ifs), std::istreambuf_iterator<char>());
	std::cout << std::endl << Text << std::endl;
}

void Shell::Execute(const std::string& command) const {
	CommandTable.at(command)();
}

void Shell::operator()(const std::string& command) const {
	try { // Funktion ausfhren
		Execute(command);
	}
	catch(...) {
		try { // Text anzeigen
			displayText(command);
		}
		catch(...) {
			std::cout << "Den Befehl \"" << command << "\" gibt es nicht.";
		}
	}
}

const Shell& Shell::operator<<(const std::string& string) const {
	std::cout << std::endl << std::endl << string << std::endl << std::endl;
	return(*this);
}

void Shell::AddCommand(const char* command, std::function<void(void)> func) {
	CommandTable[command] = func;
}
