#include "Ressourcen.h"
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <exception>


Resource::Resource(Resource&& r)
	: type(r.type)
{
	switch (type) {
		case IMAGE:
			image = r.image;
			r.image = nullptr;
			break;
		case TEXTURE:
			texture = r.texture;
			r.texture = nullptr;
			break;
		case SOUND:
			sound = r.sound;
			r.sound = nullptr;
			break;
		case FONT:
			font = r.font;
			r.font = nullptr;
			break;
		default:
			break;
	}
	r.type = NONE;
}

Resource& Resource::operator=(Resource&& r)
{
	switch (type) {
	case IMAGE:
		image = r.image;
		r.image = nullptr;
		break;
	case TEXTURE:
		texture = r.texture;
		r.texture = nullptr;
		break;
	case SOUND:
		sound = r.sound;
		r.sound = nullptr;
		break;
	case FONT:
		font = r.font;
		r.font = nullptr;
		break;
	default:
		break;
	}
	type = r.type;
	r.type = NONE;
	return *this;
}

template<>
UTIL_API void Resource::load<sf::Texture>(const std::string& path) {
	if(type == NONE)
	{
		type = TEXTURE;
		texture = new sf::Texture();
		texture->loadFromFile(path);
		texture->setSmooth(true);
	}
}
template<>
UTIL_API void Resource::load<sf::Image>(const std::string& path) {
	if(type == NONE)
	{
		type = IMAGE;
		image = new sf::Image();
		image->loadFromFile(path);
	}
}
template<>
UTIL_API void Resource::load<sf::SoundBuffer>(const std::string& path) {
	if(type == NONE)
	{
		type = SOUND;
		sound = new sf::SoundBuffer();
		sound->loadFromFile(path);
	}
}
template<>
UTIL_API void Resource::load<sf::Font>(const std::string& path) {
	if(type == NONE)
	{
		type = FONT;
		font = new sf::Font();
		font->loadFromFile(path);
	}
}

template<>
UTIL_API sf::Texture& Resource::get<sf::Texture>() const {
	if (type != TEXTURE)
		throw std::logic_error("Wrong resource type!");
	return *texture;
}
template<>
UTIL_API sf::Image& Resource::get<sf::Image>() const {
	if (type != IMAGE)
		throw std::logic_error("Wrong resource type!");
	return *image;
}
template<>
UTIL_API sf::SoundBuffer& Resource::get<sf::SoundBuffer>() const {
	if (type != SOUND)
		throw std::logic_error("Wrong resource type!");
	return *sound;
}
template<>
UTIL_API sf::Font& Resource::get<sf::Font>() const {
	if (type != FONT)
		throw std::logic_error("Wrong resource type!");
	return *font;
}

Resource::~Resource() {
	switch(type) {
		case IMAGE:
			delete image;
			break;
		case TEXTURE:
			delete texture;
			break;
		case SOUND:
			delete sound;
			break;
		case FONT:
			delete font;
			break;
		default:
			break;
	}
}

template<typename T>
UTIL_API T& Resourcemanager::get(const std::string& path) {
	auto it = resourcen.find(path);
	if (it != resourcen.end())
		return it->second.get<T>();

	resourcen[path].load<T>(basePath + path);
	return resourcen[path].get<T>();
}
template<typename T>
UTIL_API T& Resourcemanager::get(const char* path) {
	return get<T>(std::string(path));
}

template<>
UTIL_API sf::Image& Resourcemanager::get<sf::Image>(const std::string& path) {
	std::string searchpath = "i_" + path;
	auto it = resourcen.find(searchpath);
	if (it != resourcen.end())
		return it->second.get<sf::Image>();

	resourcen[searchpath].load<sf::Image>(basePath + path);
	return resourcen[searchpath].get<sf::Image>();
}

template UTIL_API sf::Texture& Resourcemanager::get<sf::Texture>(const std::string& path);
template UTIL_API sf::Font& Resourcemanager::get<sf::Font>(const std::string& path);
template UTIL_API sf::SoundBuffer& Resourcemanager::get<sf::SoundBuffer>(const std::string& path);
template UTIL_API sf::Image& Resourcemanager::get<sf::Image>(const char* path);
template UTIL_API sf::Texture& Resourcemanager::get<sf::Texture>(const char* path);
template UTIL_API sf::Font& Resourcemanager::get<sf::Font>(const char* path);
template UTIL_API sf::SoundBuffer& Resourcemanager::get<sf::SoundBuffer>(const char* path);
