#ifndef RESSOURCEN_H
#define RESSOURCEN_H

#include "Config.h"
#include "Util.h"
#include <unordered_map>
#include <string>


namespace sf {
	class Image;
	class Texture;
	class SoundBuffer;
	class Font;
}

class UTIL_API Resource {
	enum Type {NONE, TEXTURE, IMAGE, SOUND, FONT};
	Type type;
	union {
		sf::Image* image;
		sf::Texture* texture;
		sf::SoundBuffer* sound;
		sf::Font* font;
	};

	Resource(const Resource&) = delete;
	Resource& operator=(const Resource&) = delete;
public:
	// Default constructor
	//______________________
	Resource() : type(NONE) {}

	// Move constructor
	//______________________
	Resource(Resource&& r);

	// Move operator
	//______________________
	Resource& operator=(Resource&& r);

	// Loads a resource. Specialized for the different kinds of resources.
	//	path: Path of the resource to be loaded
	//______________________
	template<typename T>
	void load(const std::string& path);

	// Returns a reference to the resource. Specialized for the different kinds of resources
	//______________________
	template<typename T>
	T& get() const;

	// Destructor
	//______________________
	~Resource();
};


class UTIL_API Resourcemanager {
	std::unordered_map<std::string, Resource> resourcen;
	std::string basePath;

	Resourcemanager(const Resourcemanager&) = delete;
	void operator=(const Resourcemanager&) = delete;
public:
	explicit Resourcemanager(const std::string& basePath_ = std::emptyString) : basePath(basePath_) {}

	// Returns a reference to the resource object. Takes the type of the resource as template parameter. If the resource object is not already loaded from file, it is loaded
	//	path: Path of the resource
	//______________________
	template<typename T>
	UTIL_API T& get(const std::string& path);
	template<typename T>
	UTIL_API T& get(const char* path);
};


#endif
