#include "Kommunikator.h"
#include <functional>
#include <iostream>
#include "SoundLoop.h"


Kommunikator Client;

void ExitMatch() {
	Client.ExitMatch();
}
void Reconnect() {
	Client.Disconnect();
	Client.Anmelden();
}


Kommunikator::Kommunikator()
	: sf::Thread(std::bind(&Kommunikator::Run, this))
{
	//Anmelden(); //Funzt warum auch immer nicht...
	Client.setBlocking(true);
	launch();
}

void Kommunikator::Anmelden(sf::IpAddress IP, int Port) {
	if(Client.connect(IP, Port) != sf::Socket::Done) {
		std::cout << "Konnte nicht verbunden werden...";
	}
}

void Kommunikator::Run() {
	std::srand(static_cast<unsigned int>(std::time(nullptr)));
	unsigned int msgID, c;
	std::string msg;
	unsigned char a, b;
	bool d;
	while(GetStatus() != ENDE) {
		iMeldung.clear();
		Client.receive(iMeldung);
		iMeldung >> msgID;
		switch(msgID) {
			case Welcome:
				iMeldung >> msg >> a >> b;
				std::cout << msg << "\n"
					<< "Spielfeldgroesse: " << static_cast<int>(a) << "\n"
					<< "Anzahl Schiffe: " << static_cast<int>(b) << "\n";
				break;
			case Error:
				iMeldung >> c >> msg;
				if(c == AlreadyBombed && KISpielt) {
					WerfeRandomBombe();
					break;
				}
				else {
					std::string fString = "Fehler (" + std::to_string(c) + "): " + msg;
					SetFehler(fString);
					std::cout << fString;
				}
				break;
			case MatchInfoList:
				iMeldung >> c;
				for(unsigned int i = 0; i < c; i++) {
					iMeldung >> d;
					iMeldung >> msg;
					if(d) {
						Spielauswahl.emplace_back("Gegner: " + msg, sf::Vector2f(25, 55+i*30), sf::Vector2f(550, 80+i*30), 15, 2.f);
					}
					else {
						iMeldung >> msg;
					}
				}
				SetStatus(AUSWAHL);
				ResetFehler();
				break;
			case AuthSuccessful:
				SetStatus(MEN);
				break;
			case MatchJoined:
				iMeldung >> msg;
				if (!msg.empty()) {
					Gegner.setColor(sf::Color(0,0,0));
					Gegner.setPosition(600, 110);
					Gegner.setString("Gegner:\n  " + msg);
					SetStatus(SetzeN);
					if(KISpielt) {
						SetRandomSchiffe();
					}
				}
				break;
			case Matchleft:
				if(GetStatus() != GEWONNEN && GetStatus() != VERLOREN) {
					SoundPlayer.PlaySound(FEIND_WEG);
					SetStatus(MEN);
				}
				break;
			case PlaceShip:
				break;
			case PlacementComplete:
				SoundPlayer.PlaySound(POSITION_EINGENOMMEN);
				SetStatus(GESETZT);
				break;
			case GameStarts:
				SetStatus(WERFEN);
				break;
			case YourTurn:
				AnDerReihe.setString("Sie knnen nun Angreifen!");
				AnDerReihe.setPosition(400-AnDerReihe.getLocalBounds().width/2,435);
				AmZug = true;
				if(KISpielt) {
					WerfeRandomBombe();
				}
				break;
			case YourTurnEnded:
				AnDerReihe.setString("Bitte warten Sie auf den gegnerischen Angriff!");
				AnDerReihe.setPosition(400-AnDerReihe.getLocalBounds().width/2,435);
				AmZug = false;
				break;
			case BombHit:
				iMeldung >> a >> b;
				if(AmZug) {
					SoundPlayer.PlaySound(SCHIFF_Getroffen);
					(*Treffer)(a, b, true);
				}
				else {
					(*Getroffen)(a, b, true);
				}
				break;
			case BombMissed:
				iMeldung >> a >> b;
				if(AmZug) {
					(*Treffer)(a, b, false);
				}
				else {
					(*Getroffen)(a, b, false);
				}
				break;
			case YouLose:
				SetStatus(VERLOREN);
				break;
			case YouWin:
				SetStatus(GEWONNEN);
				break;
		}
	}
}

void Kommunikator::SetNick(const std::string& Nick) {
	oMeldung.clear();
	oMeldung << ChangeNickname << Nick;
	Client.send(oMeldung);
}

void Kommunikator::Bombardieren(unsigned char x, unsigned char y) {
	oMeldung.clear();
	oMeldung << DropBomb << x << y;
	Client.send(oMeldung);
}

void Kommunikator::SetzeSchiff(unsigned char Num, bool Vertikal, const sf::Vector2<unsigned char>& Koord) {
	oMeldung.clear();
	oMeldung << PlaceShip << Num << Koord.x << Koord.y;
	if(Vertikal) {
		oMeldung << static_cast<unsigned char>(1);
	}
	else {
		oMeldung << static_cast<unsigned char>(0);
	}
	Client.send(oMeldung);
}

void Kommunikator::ExitMatch() {
	oMeldung.clear();
	oMeldung << LeaveMatch;
	Client.send(oMeldung);
}

void Kommunikator::ZuflligesSpiel() {
	oMeldung.clear();
	oMeldung << JoinRandomMatch;
	Client.send(oMeldung);
	SetStatus(WARTEN);
}

void Kommunikator::NeuesSpiel() {
	oMeldung.clear();
	oMeldung << CreateMatch;
	Client.send(oMeldung);
	SetStatus(WARTEN);
	SetFehler("Ein neues Spiel wurde erffnet... Es wird auf einen Gegner gewartet...");
}

void Kommunikator::BestimmtesSpiel(const std::string& gegner) {
	oMeldung.clear();
	oMeldung << JoinMatch << gegner;
	Client.send(oMeldung);
	SetStatus(WARTEN);
	SetFehler("Bitte warten... Sie werden mit dem gewnschten Spiel verbunden");
}

void Kommunikator::MatchInfo() {
	oMeldung.clear();
	SetFehler("Bitte warten... Die bersicht wird erstellt...");
	oMeldung << ReqMatchInfoList;
	Client.send(oMeldung);
	SetStatus(WARTEN);
}

void Kommunikator::Disconnect() {
	oMeldung.clear();
	oMeldung << Quit;
	Client.send(oMeldung);
}

Kommunikator::~Kommunikator() {
	Disconnect();
}
